@@ -1,10 +1,10 @@
-/* $Id: RTErrConvertFromErrno.cpp 64343 2010-08-02 15:36:30Z klaus $ */
+/* $Id: RTErrConvertFromErrno.cpp 77339 2012-04-08 16:28:24Z bird $ */
 /** @file
  * IPRT - Convert errno to iprt status codes.
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -32,16 +32,7 @@
 #include "internal/iprt.h"
 
 #include <iprt/assert.h>
-
-#if defined(RT_OS_DARWIN) && defined(KERNEL)
-# include <sys/errno.h>
-#elif defined(RT_OS_LINUX) && defined(__KERNEL__)
-# include <linux/errno.h>
-#elif defined(RT_OS_FREEBSD) && defined(_KERNEL)
-# include <sys/errno.h>
-#else
-# include <errno.h>
-#endif
+#include <iprt/errno.h>
 
 
 RTDECL(int)  RTErrConvertFromErrno(unsigned uNativeCode)
